@@ -1,4 +1,5 @@
-#include <SWI-cpp.h>
+#include <SWI-cpp2.h>
+#include <SWI-cpp2.cpp>
 #include "Rcpp.h"
 using namespace Rcpp ;
 
@@ -6,64 +7,94 @@ using namespace Rcpp ;
 //
 // [] -> NULL
 // real -> NumericVector
-// #(r1, r2, r3) -> NumericVector
+// #(r1, r2, r3) -> NumericVector (# is a default, see option realvec)
+// ##(#(row1), #(row2), ...) -> Matrix
 // integer -> IntegerVector
-// %(i1, i2, i3) -> IntegerVector
+// %(i1, i2, i3) -> IntegerVector (see option intvec for the name)
+// %%(%(row1), %(row2), ...) -> Matrix
 // string -> CharacterVector
 // $$(s1, s2, s3) CharacterVector
+// $$$($$(row1), $$(row2), ...) -> Matrix
 // na (atom) -> NA
 // true, false (atoms) -> LogicalVector
-// !(l1, l2, l3) -> LogicalVector
-// the empty atom -> empty R symbol (see below)
+// !(l1, l2, l3) -> LogicalVector (see option boolvec)
+// !!(!(row1), !(row2), ...) -> Matrix
+// the empty atom -> ""
 // other atoms -> symbol/name
 // variable -> expression(variable name)
 // compound -> call (aka. "language")
 // list -> list
-// ##, %%, $$$, !! -> R matrices
 //
 RObject pl2r(PlTerm pl, CharacterVector& names, PlTerm& vars) ;
 
 // Translate R expression to prolog
 //
 // NULL -> []
-// numeric vector of length 1 -> real
-// numeric vector of length > 1 -> e.g. #(1.0, 2.0, 3.0)
+// numeric vector of length 1 -> real (unless rolog.scalar == FALSE)
+// numeric vector of length > 1 -> e.g., #(1.0, 2.0, 3.0) (see rolog.realvec)
 // integer vector of length 1 -> integer
 // integer vector of length > 1 -> %(1, 2, 3)
 // character vector of length 1 -> string
-// character vector of length > 1 -> $$("a", "b", "c")
+// character vector of length > 1 -> $("a", "b", "c")
 // logical vector of length 1 -> the atoms true, false or na
-// logical vector of length > 1 -> !(true, false, na)
-// matrices -> ##, %%, $$$, !! with vectors as elements
+// logical vector of length > 1 -> $(true, false, na)
 // other symbols/name -> atom
 // expression -> variable
 // call/language -> compound
 // list -> list
-// functions -> :-/2
 //
 PlTerm r2pl(SEXP r, CharacterVector& names, PlTerm& vars) ;
 
+// Consult one or more files. If something fails, the procedure stops, and
+// will not try to consult the remaining files.
+//
+// [[Rcpp::export(.consult)]]
+LogicalVector consult_(CharacterVector files)
+{
+  for(R_xlen_t i=0; i<files.size(); i++)
+  {
+    try 
+    {
+      PlCall("consult", PlTermv(PlTerm_string(files(i))));
+    }
+    
+    catch(PlException& ex)
+    { 
+      String err(ex.as_string(PlEncoding::Locale));
+      PL_clear_exception() ;
+      stop("failed to consult %s: %s", (char*) files(i), err.get_cstring()) ;
+    }
+  }
+
+  return true ;
+}
+
 // Prolog -> R
 RObject pl2r_null()
 {
   return R_NilValue ;
 }
 
+// TODO: use this and pl.eq_if_atom(ATOM_na) instead of
+//       pl.is_atom() && pl == "na"
+// PlAtom ATOM_na("na");
+
 // This helper function checks for na and then translates an individual PlTerm 
 // to a double.
 double pl2r_double(PlTerm pl)
 {
-  if(PL_is_atom(pl) && pl == "na")
+  if(pl.is_atom() && pl.as_string() == "na")
     return NA_REAL ;
 
   try 
   {
-    return (double) pl ;
+    return pl.as_float() ;
   }
 
   catch(PlException& ex)
   { 
-    warning("cannot convert %s to float: %s", (char*) pl, (char*) ex) ;
+    warning("cannot convert %s to float: %s",
+            pl.as_string(PlEncoding::Locale).c_str(), ex.as_string(PlEncoding::Locale).c_str()) ;
     PL_clear_exception() ;
     return NA_REAL ;
   }
@@ -80,7 +111,7 @@ DoubleVector pl2r_realvec(PlTerm pl)
 {
   DoubleVector r(pl.arity()) ;
   for(size_t i=0; i<pl.arity(); i++)
-    r(i) = pl2r_double(pl.operator[](i+1)) ;
+    r(i) = pl2r_double(pl[i+1]) ;
 
   return r ;
 }
@@ -112,17 +143,18 @@ NumericMatrix pl2r_realmat(PlTerm pl)
 // See above for pl2r_double
 long pl2r_int(PlTerm pl)
 {
-  if(PL_is_atom(pl) && pl == "na")
+  if(pl.is_atom() && pl.as_string() == "na")
     return NA_INTEGER ;
 
   try 
   {
-    return (long) pl ;
+    return pl.as_long() ;
   }
   
   catch(PlException& ex)
   { 
-    warning("Cannot convert %s to integer: %s", (char*) pl, (char*) ex) ;
+    warning("Cannot convert %s to integer: %s",
+            pl.as_string(PlEncoding::Locale).c_str(), ex.as_string(PlEncoding::Locale).c_str()) ;
     PL_clear_exception() ;
     return NA_INTEGER ;
   }
@@ -137,7 +169,7 @@ IntegerVector pl2r_intvec(PlTerm pl)
 {
   IntegerVector r(pl.arity()) ;
   for(size_t i=0; i<pl.arity(); i++)
-    r(i) = pl2r_int(pl.operator[](i+1)) ;
+    r(i) = pl2r_int(pl[i+1]) ;
 
   return r ;
 }
@@ -168,10 +200,10 @@ IntegerMatrix pl2r_intmat(PlTerm pl)
 // See above for pl2r_double
 String pl2r_string(PlTerm pl)
 {
-  if(PL_is_atom(pl) && pl == "na")
+  if(pl.is_atom() && pl.as_string() == "na")
     return NA_STRING ;
   
-  return (char*) pl ;
+  return pl.as_string(PlEncoding::Locale) ;
 }
 
 CharacterVector pl2r_char(PlTerm pl)
@@ -183,7 +215,7 @@ CharacterVector pl2r_charvec(PlTerm pl)
 {
   CharacterVector r(pl.arity()) ;
   for(size_t i=0; i<pl.arity(); i++)
-    r(i) = pl2r_string(pl.operator[](i+1)) ;
+    r(i) = pl2r_string(pl[i+1]) ;
 
   return r ;
 }
@@ -214,20 +246,57 @@ CharacterMatrix pl2r_charmat(PlTerm pl)
 // Convert prolog atom to R symbol (handle na, true, false)
 RObject pl2r_symbol(PlTerm pl)
 {
-  if(pl == "na")
+  if(pl.as_string() == "na")
     return wrap(NA_LOGICAL) ;
   
-  if(pl == "true")
+  if(pl.as_string() == "true")
     return wrap(true) ;
   
-  if(pl == "false")
+  if(pl.as_string() == "false")
     return wrap(false) ;
 
-  // empty symbols
-  if(pl == "")
+  // Empty symbols
+  if(pl.as_string() == "")
     return Function("substitute")() ;
 
-  return as<RObject>(Symbol((char*) pl)) ;
+  return as<RObject>(Symbol(pl.as_string(PlEncoding::UTF8))) ; // TODO: PlEncoding::Locale?
+}
+
+// Forward declaration, needed below
+RObject pl2r_compound(PlTerm pl, CharacterVector& names, PlTerm& vars) ;
+
+// Convert prolog neck to R function
+RObject pl2r_function(PlTerm pl, CharacterVector& names, PlTerm& vars)
+{
+  PlTerm plhead = pl[1] ;
+  PlTerm plbody = pl[2] ;
+
+  Language head("alist") ;
+  for(unsigned int i=1 ; i<=plhead.arity() ; i++)
+  {
+    PlTerm arg = plhead[i] ;
+
+    // Compounds like mean=100 are translated to named function arguments
+    if(arg.is_compound() && arg.name().as_string() == "=" && arg.arity() == 2)
+    {
+      PlTerm a1 = arg[1] ;
+      PlTerm a2 = arg[2] ;
+      if(a1.is_atom())
+      {
+        head.push_back(Named(a1.as_string(PlEncoding::UTF8)) = pl2r(a2, names, vars)) ;
+        continue ;
+      }
+    }
+
+    // the argument is the name
+    head.push_back(Named(arg.as_string(PlEncoding::UTF8)) = pl2r_symbol(PlTerm_atom(""))) ;
+  }
+
+  RObject body = pl2r_compound(plbody, names, vars) ;
+  head.push_back(body) ;
+
+  Function as_function("as.function") ;
+  return wrap(as_function(head)) ;
 }
 
 LogicalVector pl2r_boolvec(PlTerm pl)
@@ -235,29 +304,29 @@ LogicalVector pl2r_boolvec(PlTerm pl)
   LogicalVector r(pl.arity()) ;
   for(size_t i=0; i<pl.arity(); i++)
   {
-    PlTerm t = pl.operator[](i+1) ;
-    if(PL_is_atom(t))
+    PlTerm t = pl[i+1] ;
+    if(t.is_atom())
     {
-      if(t == "na")
+      if(t.as_string() == "na")
       {
         r(i) = NA_LOGICAL ;
         continue ;
       }
       
-      if(t == "true")
+      if(t.as_string() == "true")
       {
         r(i) = 1 ;
         continue ;
       }
       
-      if(t == "false")
+      if(t.as_string() == "false")
       {
         r(i) = 0 ;
         continue ;
       }
     }
     
-    warning("r2pl_logical: invalid item %s, returning NA", (char*) t) ;
+    warning("r2pl_logical: invalid item %s, returning NA", t.as_string(PlEncoding::Locale).c_str()) ;
     r(i) = NA_LOGICAL ;
   }
 
@@ -287,43 +356,6 @@ LogicalMatrix pl2r_boolmat(PlTerm pl)
   return r ;
 }
 
-// needed below
-RObject pl2r_compound(PlTerm pl, CharacterVector& names, PlTerm& vars) ;
-
-// Convert prolog neck to R function
-RObject pl2r_function(PlTerm pl, CharacterVector& names, PlTerm& vars)
-{
-  PlTerm plhead = pl[1] ;
-  PlTerm plbody = pl[2] ;
-
-  Language head("alist") ;
-  for(unsigned int i=1 ; i<=plhead.arity() ; i++)
-  {
-    PlTerm arg = plhead[i] ;
-
-    // Compounds like mean=100 are translated to named function arguments
-    if(PL_is_compound(arg) && !strcmp(arg.name(), "=") && arg.arity() == 2)
-    {
-      PlTerm a1 = arg.operator[](1) ;
-      PlTerm a2 = arg.operator[](2) ;
-      if(PL_is_atom(a1))
-      {
-        head.push_back(Named(a1.name()) = pl2r(a2, names, vars)) ;
-        continue ;
-      }
-    }
-
-    // the argument is the name
-    head.push_back(Named((char*) arg) = pl2r_symbol(PlAtom(""))) ;
-  }
-
-  RObject body = pl2r_compound(plbody, names, vars) ;
-  head.push_back(body) ;
-
-  Function as_function("as.function") ;
-  return wrap(as_function(head)) ;
-}
-
 // Translate prolog variables to R expressions.
 RObject pl2r_variable(PlTerm pl, CharacterVector& names, PlTerm& vars)
 {
@@ -333,19 +365,19 @@ RObject pl2r_variable(PlTerm pl, CharacterVector& names, PlTerm& vars)
   //
   // Search for the variable (e.g., _1545) in names and return its R name as an
   // expression (say, X).
-  PlTail tail(vars) ;
-  PlTerm v ;
+  PlTerm_tail tail(vars) ;
+  PlTerm_var v ;
   for(int i=0 ; i<names.length() ; i++)
   {
-    tail.next(v) ;
-    if(!strcmp(v, pl))
+    PlCheckFail(tail.next(v)) ;
+    if(v == pl)
       return ExpressionVector::create(Symbol(names(i))) ;
   }
   
   // If the variable is not found, it's a new one created by Prolog, e.g., in
   // queries like member(1, Y), Y is unified with [1 | _NewVar ]. This variable
   // cannot be translated to a human-readable name, so it is returned as _1545.
-  return ExpressionVector::create(Symbol((char*) pl)) ;
+  return ExpressionVector::create(Symbol(pl.as_string(PlEncoding::UTF8))) ; // TODO: PlEncoding::Locale?
 }
 
 // Translate prolog compound to R call
@@ -356,59 +388,59 @@ RObject pl2r_variable(PlTerm pl, CharacterVector& names, PlTerm& vars)
 RObject pl2r_compound(PlTerm pl, CharacterVector& names, PlTerm& vars)
 {
   // This function does not (yet) work for cyclic terms
-  if(!PL_is_acyclic(pl))
-    stop("pl2r: Cannot convert cyclic term %s", (char*) pl) ;
+  if(!PL_is_acyclic(pl.C_))
+    stop("pl2r: Cannot convert cyclic term %s", pl.as_string(PlEncoding::Locale).c_str()) ;
 
   // Convert ##(#(...), ...) to NumericMatrix
-  if(!strcmp(pl.name(), "##"))
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "###"))
     return pl2r_realmat(pl) ;
 
-  // Convert #(1.0, 2.0, 3.0) to DoubleVectors
-  if(!strcmp(pl.name(), "#"))
+  // Convert #(1.0, 2.0, 3.0) to DoubleVector
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "##"))
     return pl2r_realvec(pl) ;
 
   // Convert %%(%(...), ...) to IntegerMatrix
-  if(!strcmp(pl.name(), "%%"))
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "%%%"))
     return pl2r_intmat(pl) ;
 
-  // Convert %(1.0, 2.0, 3.0) to IntegerVectors
-  if(!strcmp(pl.name(), "%"))
+  // Convert %(1.0, 2.0, 3.0) to IntegerVector
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "%%"))
     return pl2r_intvec(pl) ;
 
   // Convert $$$($$(...), ...) to StringMatrix
-  if(!strcmp(pl.name(), "$$$"))
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "$$$"))
     return pl2r_charmat(pl) ;
 
-  // Convert $(1.0, 2.0, 3.0) to CharacterVectors
-  if(!strcmp(pl.name(), "$$"))
+  // Convert $$(1.0, 2.0, 3.0) to CharacterVector
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "$$"))
     return pl2r_charvec(pl) ;
 
   // Convert !!(!(...), ...) to LogicalMatrix
-  if(!strcmp(pl.name(), "!!"))
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "!!!"))
     return pl2r_boolmat(pl) ;
 
-  // Convert !(1.0, 2.0, 3.0) to LogicalVectors
-  if(!strcmp(pl.name(), "!"))
+  // Convert !(1.0, 2.0, 3.0) to LogicalVector
+  if(!strcmp(pl.name().as_string(PlEncoding::UTF8).c_str(), "!!"))
     return pl2r_boolvec(pl) ;
 
   // Convert :- to function
-  if(!strcmp(pl.name(), ":-"))
+  if(pl.name().as_string() == ":-")
     return pl2r_function(pl, names, vars) ;
 
   // Other compounds
-  Language r(pl.name()) ;
+  Language r(pl.name().as_string(PlEncoding::UTF8).c_str()) ;
   for(unsigned int i=1 ; i<=pl.arity() ; i++)
   {
     PlTerm arg = pl[i] ;
 
     // Compounds like mean=100 are translated to named function arguments
-    if(PL_is_compound(arg) && !strcmp(arg.name(), "=") && arg.arity() == 2)
+    if(arg.is_compound() && !strcmp(arg.name().as_string(PlEncoding::UTF8).c_str(), "=") && arg.arity() == 2)
     {
-      PlTerm a1 = arg.operator[](1) ;
-      PlTerm a2 = arg.operator[](2) ;
-      if(PL_is_atom(a1))
+      PlTerm a1 = arg[1] ;
+      PlTerm a2 = arg[2] ;
+      if(a1.is_atom())
       {
-        r.push_back(Named(a1.name()) = pl2r(a2, names, vars)) ;
+        r.push_back(Named(a1.name().as_string(PlEncoding::UTF8).c_str()) = pl2r(a2, names, vars)) ;
         continue ;
       }
     }
@@ -423,7 +455,7 @@ RObject pl2r_compound(PlTerm pl, CharacterVector& names, PlTerm& vars)
 // Translate prolog list to R list
 //
 // This code allows for lists like [1, 2 | Tail] with variable tail. These 
-// cannot be processed by PlTail, therefore, the code is a bit more 
+// cannot be processed by PlTerm_tail, therefore, the code is a bit more 
 // complicated, also because it can handle named arguments.
 //
 // Examples:
@@ -433,22 +465,22 @@ RObject pl2r_compound(PlTerm pl, CharacterVector& names, PlTerm& vars)
 //
 RObject pl2r_list(PlTerm pl, CharacterVector& names, PlTerm& vars)
 {
-  PlTerm head = pl.operator[](1) ;
+  PlTerm head = pl[1] ;
   
   // if the tail is a list or empty, return a normal list
-  RObject tail = pl2r(pl.operator[](2), names, vars) ;
+  RObject tail = pl2r(pl[2], names, vars) ;
   if(TYPEOF(tail) == VECSXP || TYPEOF(tail) == NILSXP)
   {
     List r = as<List>(tail) ;
     
     // convert prolog pair a-X to named list element
-    if(PL_is_compound(head) && !strcmp(head.name(), "-") && head.arity() == 2)
+    if(head.is_compound() && !strcmp(head.name().as_string(PlEncoding::UTF8).c_str(), "-") && head.arity() == 2)
     {
-      PlTerm a1 = head.operator[](1) ;
-      PlTerm a2 = head.operator[](2) ;
-      if(PL_is_atom(a1))
+      PlTerm a1 = head[1] ;
+      PlTerm a2 = head[2] ;
+      if(a1.is_atom())
       {
-        r.push_front(pl2r(a2, names, vars), a1.name()) ;
+        r.push_front(pl2r(a2, names, vars), a1.name().as_string(PlEncoding::UTF8).c_str()) ;
         return r ;
       }
     }
@@ -459,16 +491,16 @@ RObject pl2r_list(PlTerm pl, CharacterVector& names, PlTerm& vars)
   }
     
   // if the tail is something else, return [|](head, tail)
-  Language r(pl.name()) ;
+  Language r(pl.name().as_string(PlEncoding::UTF8).c_str()) ;
   
   // convert prolog pair a-X to named list element
-  if(PL_is_compound(head) && !strcmp(head.name(), "-") && head.arity() == 2)
+  if(head.is_compound() && !strcmp(head.name().as_string(PlEncoding::UTF8).c_str(), "-") && head.arity() == 2)
   {
-    PlTerm a1 = head.operator[](1) ;
-    PlTerm a2 = head.operator[](2) ;
-    if(PL_is_atom(a1))
+    PlTerm a1 = head[1] ;
+    PlTerm a2 = head[2] ;
+    if(a1.is_atom())
     {
-      r.push_back(Named(a1.name()) = pl2r(a2, names, vars)) ;
+      r.push_back(Named(a1.name().as_string(PlEncoding::UTF8).c_str()) = pl2r(a2, names, vars)) ;
       r.push_back(tail) ;
       return as<RObject>(r) ;
     }
@@ -482,53 +514,53 @@ RObject pl2r_list(PlTerm pl, CharacterVector& names, PlTerm& vars)
 
 RObject pl2r(PlTerm pl, CharacterVector& names, PlTerm& vars)
 {
-  if(PL_term_type(pl) == PL_NIL)
+  if(pl.type() == PL_NIL)
     return pl2r_null() ;
   
-  if(PL_is_integer(pl))
+  if(pl.is_integer())
     return pl2r_integer(pl) ;
   
-  if(PL_is_float(pl))
+  if(pl.is_float())
     return pl2r_real(pl) ;
   
-  if(PL_is_string(pl))
+  if(pl.is_string())
     return pl2r_char(pl) ;
   
-  if(PL_is_atom(pl))
+  if(pl.is_atom())
     return pl2r_symbol(pl) ;
   
-  if(PL_is_list(pl))
+  if(pl.is_list())
     return pl2r_list(pl, names, vars) ;
   
-  if(PL_is_compound(pl))
+  if(pl.is_compound())
     return pl2r_compound(pl, names, vars) ;
   
-  if(PL_is_variable(pl))
+  if(pl.is_variable())
     return pl2r_variable(pl, names, vars) ;
   
-  stop("pl2r: Cannot convert %s", (char*) pl) ;
+  stop("pl2r: Cannot convert %s", pl.as_string(PlEncoding::Locale).c_str()) ;
 }
 
 // Translate R expression to prolog
 //
 // Forward declarations
-PlTerm r2pl_real(NumericVector r, bool ascalar=TRUE) ;
-PlTerm r2pl_logical(LogicalVector r, bool ascalar=TRUE) ;
-PlTerm r2pl_integer(IntegerVector r, bool ascalar=TRUE) ;
-PlTerm r2pl_string(CharacterVector r, bool ascalar=TRUE) ;
+PlTerm r2pl_real(NumericVector r) ;
+PlTerm r2pl_logical(LogicalVector r) ;
+PlTerm r2pl_integer(IntegerVector r) ;
+PlTerm r2pl_string(CharacterVector r) ;
 
 // This returns an empty list
 PlTerm r2pl_null()
 {
-  PlTerm pl ;
-  PlTail(pl).close() ;
+  PlTerm_var pl ;
+  PlCheckFail(PlTerm_tail(pl).close()) ;
   return pl ;
 }
 
 // Prolog representation of R's NA.
 PlTerm r2pl_na()
 {
-  return PlAtom("na") ;
+  return PlTerm_atom("na") ;
 }
 
 // Translate to matrix ##(#(1.0, 2.0, 3.0), #(4.0, 5.0, 6.0))
@@ -536,13 +568,13 @@ PlTerm r2pl_matrix(Matrix<REALSXP> r)
 {
   PlTermv rows(r.nrow()) ;
   for(int i=0 ; i<r.nrow() ; i++)
-    rows[i] = r2pl_real(r.row(i), FALSE) ;
+    PlCheckFail(rows[i].unify_term(r2pl_real(r.row(i)))) ;
 
-  return PlCompound("##", rows) ;
+  return PlCompound("###", rows) ;
 }
 
 // Translate to (scalar) real or compounds like #(1.0, 2.0, 3.0)
-PlTerm r2pl_real(NumericVector r, bool ascalar)
+PlTerm r2pl_real(NumericVector r)
 {
   if(Rf_isMatrix(r))
     return r2pl_matrix(as<Matrix<REALSXP>>(r)) ;
@@ -554,14 +586,13 @@ PlTerm r2pl_real(NumericVector r, bool ascalar)
   LogicalVector na = is_na(r) ;
   
   // Translate to scalar
-  if(ascalar)
-    if(r.length() == 1)
-    {
-      if(na[0] && !nan[0])
-        return r2pl_na() ;
+  if(r.length() == 1)
+  {
+    if(na[0] && !nan[0])
+      return r2pl_na() ;
     
-      return PlTerm((double) r[0]) ;
-    }
+    return PlTerm_float(r[0]);
+  }
 
   // Translate to vector #(1.0, 2.0, 3.0)
   size_t len = (size_t) r.length() ;
@@ -569,27 +600,26 @@ PlTerm r2pl_real(NumericVector r, bool ascalar)
   for(size_t i=0 ; i<len ; i++)
   {
     if(na[i] && !nan[i])
-      args[i] = r2pl_na() ;
+      PlCheckFail(args[i].unify_term(r2pl_na())) ;
     else
-      args[i] = PlTerm((double) r[i]) ;
+      PlCheckFail(args[i].unify_float(r[i])) ;
   }
-  
-  return PlCompound("#", args) ;
-}
 
+  return PlCompound("##", args) ;
+}
 
 // Translate to matrix !!(!(true, false), !(false, true))
 PlTerm r2pl_matrix(Matrix<LGLSXP> r)
 {
   PlTermv rows(r.nrow()) ;
   for(int i=0 ; i<r.nrow() ; i++)
-    rows[i] = r2pl_logical(r.row(i), FALSE) ;
+    PlCheckFail(rows[i].unify_term(r2pl_logical(r.row(i)))) ;
 
-  return PlCompound("!!", rows) ;
+  return PlCompound("!!!", rows) ;
 }
 
 // Translate to (scalar) boolean or compounds like !(true, false, na)
-PlTerm r2pl_logical(LogicalVector r, bool ascalar)
+PlTerm r2pl_logical(LogicalVector r)
 {
   if(Rf_isMatrix(r))
     return r2pl_matrix(as<Matrix<LGLSXP>>(r)) ;
@@ -600,14 +630,13 @@ PlTerm r2pl_logical(LogicalVector r, bool ascalar)
   LogicalVector na = is_na(r) ;
   
   // scalar boolean
-  if(ascalar)
-    if(r.length() == 1)
-    {
-      if(na[0])
-        return r2pl_na() ;
+  if(r.length() == 1)
+  {
+    if(na[0])
+      return r2pl_na() ;
     
-      return PlTerm(r[0] ? "true" : "false") ;
-    }
+    return PlTerm_atom(r[0] ? "true" : "false") ;
+  }
 
   // LogicalVector !(true, false, na)
   size_t len = (size_t) r.length() ;
@@ -615,12 +644,12 @@ PlTerm r2pl_logical(LogicalVector r, bool ascalar)
   for(size_t i=0 ; i<len ; i++)
   {
     if(na[i])
-      args[i] = r2pl_na() ;
+      PlCheckFail(args[i].unify_term(r2pl_na())) ;
     else
-      args[i] = PlTerm(r[i] ? "true" : "false") ;
+      PlCheckFail(args[i].unify_atom(r[i] ? "true" : "false")) ;  // TODO: unify_bool()
   }
 
-  return PlCompound("!", args) ;
+  return PlCompound("!!", args) ;
 }
 
 // Translate to matrix %%(%(1, 2), %(3, 4))
@@ -628,13 +657,13 @@ PlTerm r2pl_matrix(Matrix<INTSXP> r)
 {
   PlTermv rows(r.nrow()) ;
   for(int i=0 ; i<r.nrow() ; i++)
-    rows[i] = r2pl_integer(r.row(i), FALSE) ;
+    PlCheckFail(rows[i].unify_term(r2pl_integer(r.row(i)))) ;
 
-  return PlCompound("%%", rows) ;
+  return PlCompound("%%%", rows) ;
 }
 
 // Translate to (scalar) integer or compounds like %(1, 2, 3)
-PlTerm r2pl_integer(IntegerVector r, bool ascalar)
+PlTerm r2pl_integer(IntegerVector r)
 {
   if(Rf_isMatrix(r))
     return r2pl_matrix(as<Matrix<INTSXP>>(r)) ;
@@ -645,14 +674,13 @@ PlTerm r2pl_integer(IntegerVector r, bool ascalar)
   LogicalVector na = is_na(r) ;
   
   // scalar integer
-  if(ascalar)
-    if(r.length() == 1)
-    {
-      if(na[0])
-        return r2pl_na() ;
+  if(r.length() == 1)
+  {
+    if(na[0])
+      return r2pl_na() ;
     
-      return PlTerm((long) r(0)) ;
-    }
+    return PlTerm_integer(r(0)) ;
+  }
   
   // IntegerVector %(1, 2, 3)
   size_t len = (size_t) r.length() ;
@@ -660,12 +688,12 @@ PlTerm r2pl_integer(IntegerVector r, bool ascalar)
   for(size_t i=0 ; i<len ; i++)
   {
     if(na[i])
-      args[i] = r2pl_na() ;
+      PlCheckFail(args[i].unify_term(r2pl_na())) ;
     else
-      args[i] = PlTerm((long) r[i]) ;
+      PlCheckFail(args[i].unify_integer(r[i])) ;
   }
   
-  return PlCompound("%", args) ;
+  return PlCompound("%%", args) ;
 }
 
 // Translate R expression to prolog variable
@@ -675,6 +703,7 @@ PlTerm r2pl_integer(IntegerVector r, bool ascalar)
 // variable is encountered, its name is looked up in the list of known 
 // variables, and it is unified with it if the name is found. Otherwise, a new
 // variable is created.
+//
 PlTerm r2pl_var(ExpressionVector r, CharacterVector& names, PlTerm& vars)
 {
   // Variable name in R
@@ -682,29 +711,29 @@ PlTerm r2pl_var(ExpressionVector r, CharacterVector& names, PlTerm& vars)
   
   // Do not map the anonymous variable to a known variable name
   if(n == "_")
-    return PlTerm() ;
+    return PlTerm_var() ;
 
   // Unify with existing variable of the same name
-  PlTail tail(vars) ;
-  PlTerm v ;
+  PlTerm_tail tail(vars) ;
+  PlTerm_var v ;
   for(R_xlen_t i=0 ; i<names.length() ; i++)
   {
-    tail.next(v) ;
+    PlCheckFail(tail.next(v)) ;
     if(n == names(i))
       return v ;
   }
 
   // If no such variable exists, create a new one and remember the name
   names.push_back(n.c_str()) ;
-  PlTerm pl ;
-  tail.append(pl) ;
+  PlTerm_var pl ;
+  PlCheckFail(tail.append(pl)) ;
   return pl ;
 }
 
 // Translate R symbol to prolog atom
 PlTerm r2pl_atom(Symbol r)
 {
-  return PlAtom(r.c_str()) ;
+  return PlTerm_atom(r.c_str()) ;
 }
 
 // Translate to matrix $$$($$(1, 2), $$(3, 4))
@@ -712,13 +741,13 @@ PlTerm r2pl_matrix(Matrix<STRSXP> r)
 {
   PlTermv rows(r.nrow()) ;
   for(int i=0 ; i<r.nrow() ; i++)
-    rows[i] = r2pl_string(r.row(i), FALSE) ;
+    PlCheckFail(rows[i].unify_term(r2pl_string(r.row(i)))) ;
 
   return PlCompound("$$$", rows) ;
 }
 
 // Translate CharacterVector to (scalar) string or things like $("a", "b", "c")
-PlTerm r2pl_string(CharacterVector r, bool ascalar)
+PlTerm r2pl_string(CharacterVector r)
 {
   if(Rf_isMatrix(r))
     return r2pl_matrix(as<Matrix<STRSXP>>(r)) ;
@@ -729,14 +758,13 @@ PlTerm r2pl_string(CharacterVector r, bool ascalar)
   LogicalVector na = is_na(r) ;
   
   // scalar string
-  if(ascalar)
-    if(r.length() == 1)
-    {
-      if(na[0])
-        return r2pl_na() ;
+  if(r.length() == 1)
+  {
+    if(na[0])
+      return r2pl_na() ;
     
-      return PlString(r(0)) ;
-    }
+    return PlTerm_string(r(0)) ;
+  }
 
   // compound like $("a", "b", "c")
   size_t len = (size_t) r.length() ;
@@ -744,9 +772,9 @@ PlTerm r2pl_string(CharacterVector r, bool ascalar)
   for(size_t i=0 ; i<len ; i++)
   {
     if(na[i])
-      args[i] = r2pl_na() ;
+      PlCheckFail(args[i].unify_term(r2pl_na())) ;
     else
-      args[i] = PlString(r(i)) ;
+      PlCheckFail(args[i].unify_term(PlTerm_string(r(i)))) ; // DO NOT SUBMIT - unify_string()
   }
 
   return PlCompound("$$", args) ;
@@ -764,8 +792,8 @@ PlTerm r2pl_compound(Language r, CharacterVector& names, PlTerm& vars)
   if(len == 0)
   {
     PlTermv pl(3) ;
-    pl[1] = as<Symbol>(CAR(r)).c_str() ;
-    pl[2] = 0 ;
+    PlCheckFail(pl[1].unify_atom(as<Symbol>(CAR(r)).c_str())) ;
+    PlCheckFail(pl[2].unify_integer(0)) ;
     PlCall("compound_name_arity", pl) ;
     return pl[0] ;
   }
@@ -783,9 +811,9 @@ PlTerm r2pl_compound(Language r, CharacterVector& names, PlTerm& vars)
     
     // Convert named arguments to prolog compounds a=X
     if(n.length() && n(i) != "")
-      pl[i] = PlCompound("=", PlTermv(PlAtom(n(i)), arg)) ;
+      PlCheckFail(pl[i].unify_term(PlCompound("=", PlTermv(PlTerm_atom(n(i)), arg)))) ;
     else
-      pl[i] = arg ; // no name
+      PlCheckFail(pl[i].unify_term(arg)) ; // no name
   }
 
   return PlCompound(as<Symbol>(CAR(r)).c_str(), pl) ;
@@ -803,20 +831,20 @@ PlTerm r2pl_list(List r, CharacterVector& names, PlTerm& vars)
   if(TYPEOF(r.names()) == STRSXP)
     n = as<CharacterVector>(r.names()) ;
   
-  PlTerm pl ;
-  PlTail tail(pl) ;
+  PlTerm_var pl ;
+  PlTerm_tail tail(pl) ;
   for(R_xlen_t i=0; i<r.size() ; i++)
   {
     PlTerm arg = r2pl(r(i), names, vars) ;
     
     // Convert named argument to prolog pair a-X.
     if(n.length() && n(i) != "")
-      tail.append(PlCompound("-", PlTermv(PlAtom(n(i)), arg))) ;
+      PlCheckFail(tail.append(PlCompound("-", PlTermv(PlTerm_atom(n(i)), arg)))) ;
     else
-      tail.append(arg) ; // no name
+      PlCheckFail(tail.append(arg)) ; // no name
   }
   
-  tail.close() ;
+  PlCheckFail(tail.close()) ;
   return pl ;
 }
 
@@ -824,26 +852,26 @@ PlTerm r2pl_list(List r, CharacterVector& names, PlTerm& vars)
 PlTerm r2pl_function(Function r, CharacterVector& names, PlTerm& vars)
 {
   PlTermv fun(2) ;
-  fun[1] = r2pl_compound(BODY(r), names, vars) ;
+  PlCheckFail(fun[1].unify_term(r2pl_compound(BODY(r), names, vars))) ;
   
   List formals = as<List>(FORMALS(r)) ;
   size_t len = (size_t) formals.size() ;
   if(len == 0)
   {
     PlTermv pl(3) ;
-    pl[1] = "$function" ;
-    pl[2] = 0 ;
+    PlCheckFail(pl[1].unify_atom("$function")) ;
+    PlCheckFail(pl[2].unify_integer(0)) ;
     PlCall("compound_name_arity", pl) ;
 
-    fun[0] = pl[0] ;
+    PlCheckFail(fun[0].unify_term(pl[0])) ;
     return PlCompound(":-", fun) ;
   }
   
   CharacterVector n = formals.names() ;
   PlTermv pl(len) ;
   for(size_t i=0 ; i<len ; i++)
-    pl[i] = PlAtom(n(i)) ;
-  fun[0] = PlCompound("$function", pl) ;
+    PlCheckFail(pl[i].unify_atom(n(i))) ;
+  PlCheckFail(fun[0].unify_term(PlCompound("$function", pl))) ;
   return PlCompound(":-", fun) ;
 }
 
@@ -853,13 +881,13 @@ PlTerm r2pl(SEXP r, CharacterVector& names, PlTerm& vars)
     return r2pl_compound(r, names, vars) ;
 
   if(TYPEOF(r) == REALSXP)
-    return r2pl_real(r, TRUE) ;
+    return r2pl_real(r) ;
   
   if(TYPEOF(r) == LGLSXP)
-    return r2pl_logical(r, TRUE) ;
+    return r2pl_logical(r) ;
   
   if(TYPEOF(r) == INTSXP)
-    return r2pl_integer(r, TRUE) ;
+    return r2pl_integer(r) ;
   
   if(TYPEOF(r) == EXPRSXP)
     return r2pl_var(r, names, vars) ;
@@ -868,7 +896,7 @@ PlTerm r2pl(SEXP r, CharacterVector& names, PlTerm& vars)
     return r2pl_atom(r) ;
 
   if(TYPEOF(r) == STRSXP)
-    return r2pl_string(r, TRUE) ;
+    return r2pl_string(r) ;
 
   if(TYPEOF(r) == VECSXP)
     return r2pl_list(r, names, vars) ;
@@ -885,7 +913,7 @@ PlTerm r2pl(SEXP r, CharacterVector& names, PlTerm& vars)
 class RlQuery  
 {
   CharacterVector names ;
-  PlTerm vars ;
+  PlTerm_var vars ;
   PlQuery* qid ;
   
 public:
@@ -902,7 +930,7 @@ RlQuery::RlQuery(RObject aquery)
     qid(NULL)
 {
   PlTerm pl = r2pl(aquery, names, vars) ;
-  qid = new PlQuery("call", pl) ;
+  qid = new PlQuery("call", PlTermv(PlTerm(pl))) ;
 }
 
 RlQuery::~RlQuery()
@@ -924,9 +952,9 @@ int RlQuery::next_solution()
 
   catch(PlException& ex)
   {
-    char* s = ex ; // string is stored in a 16-ring-buffer
+    warning(ex.as_string(PlEncoding::Locale).c_str()) ;
     PL_clear_exception() ;
-    stop("Query failed: %s", s) ;
+    stop("Query failed") ;
   }
 
   return q ;
@@ -936,11 +964,11 @@ List RlQuery::bindings()
 {
   List l ;
 
-  PlTail tail(vars) ;
-  PlTerm v ;
+  PlTerm_tail tail(vars) ;
+  PlTerm_var v ;
   for(int i=0 ; i<names.length() ; i++)
   {
-    tail.next(v) ;
+    PlCheckFail(tail.next(v)) ;
     RObject r = pl2r(v, names, vars) ;
     if(TYPEOF(r) == EXPRSXP && names[i] == as<Symbol>(as<ExpressionVector>(r)[0]).c_str())
     continue ;
@@ -998,6 +1026,177 @@ RObject submit_()
   return query_id->bindings() ;
 }
 
+// Execute a query once and return conditions
+//
+// Examples:
+//
+// once(call("=", 1, 2)) -> FALSE
+// once(call("=", 1, 1)) -> empty list
+// once(call("member", 1, list(2, expression(X)))) -> list stating that X = 1
+// once(call("=", list(expression(X), expression(Y)), list(1, expression(Z))))
+//   -> list stating that X = 1 and Z = Y
+// once(call("member", 1, expression(X))) -> list stating that X = [1 | _]; 
+//   e.g., something like [|]`(1, expression(`_6330`)). This is cumbersome, any
+//   better ideas are welcome.
+//
+// [[Rcpp::export(.once)]]
+RObject once_(RObject query)
+{
+  PlFrame f ;
+  if(!query_(query))
+    stop("Could not create query.") ;
+    
+  RObject l = submit_() ;
+  clear_() ;
+  return l ;
+}
+
+// Same as once_ above, but return all solutions to a query.
+// [[Rcpp::export(.findall)]]
+List findall_(RObject query)
+{
+  PlFrame f ;
+  if(!query_(query))
+    stop("Could not create query.") ;
+    
+  List results ;
+  while(true)
+  {
+    RObject l = submit_() ;
+    if(TYPEOF(l) == LGLSXP)
+      break ;
+    
+    results.push_back(l) ;
+  }
+  
+  clear_() ;
+  return results ;
+}
+
+// Pretty print query. Maybe simplify to something like this:
+// with_output_to(string(S), write_term(member(X), [variable_names(['X'=X])])).
+//
+// [[Rcpp::export(.portray)]]
+RObject portray_(RObject query)
+{
+  if(PL_current_query() != 0)
+  {
+    warning("Closing the current query.") ;
+    clear_() ;
+  }
+
+  CharacterVector names ;
+  PlTerm_var vars ;
+  PlTermv pl(3) ;
+  PlCheckFail(pl[0].unify_term(r2pl(query, names, vars))) ;
+  PlTerm_tail tail(pl[2]) ;
+  PlCheckFail(tail.append(PlCompound("quoted", PlTermv(PlTerm_atom("false"))))) ;
+  PlCheckFail(tail.append(PlCompound("spacing", PlTermv(PlTerm_atom("next_argument"))))) ;
+  PlCheckFail(tail.close()) ;
+
+  PlFrame f ;
+  PlQuery q("term_string", pl) ;
+  try
+  {
+    if(!q.next_solution())
+      return wrap(false) ;
+  }
+  
+  catch(PlException& ex)
+  {
+    warning(ex.as_string(PlEncoding::Locale).c_str()) ;
+    PL_clear_exception() ;
+    stop("portray of %s failed.", pl[0].as_string(PlEncoding::Locale).c_str()) ;
+  }
+  
+  return pl2r(pl[1], names, vars) ;
+}
+
+// Execute a query given as a string
+//
+// Example:
+// once("use_module(library(http/html_write))")
+//
+// [[Rcpp::export(.call)]]
+RObject call_(String query)
+{
+  if(PL_current_query() != 0)
+  {
+    warning("Closing the current query.") ;
+    clear_() ;
+  }
+
+  bool r = false ;
+  try
+  {
+    r = PlCall(query.get_cstring()) ;
+  }
+  
+  catch(PlException& ex)
+  {
+    warning(ex.as_string(PlEncoding::Locale).c_str()) ;
+    PL_clear_exception() ;
+    stop("query failed: %s", query.get_cstring()) ;
+  }
+  
+  return wrap(r) ;
+}
+
+// Call R expression from Prolog
+PREDICATE(r_eval, 1)
+{
+  CharacterVector names ;
+  PlTerm_var vars ;
+  RObject Expr = pl2r(A1, names, vars) ;
+  RObject Res = Expr ;
+  try
+  {
+    Language id("identity") ;
+    id.push_back(Expr) ;
+    Res = id.eval() ;
+  }
+
+  catch(std::exception& ex)
+  {
+    throw PlException(PlCompound("r_eval", PlTermv(A1, PlTerm_atom(ex.what())))) ;
+  }
+
+  return true ;
+}
+
+// Evaluate R expression from Prolog
+PREDICATE(r_eval, 2)
+{
+  CharacterVector names ;
+  PlTerm_var vars ;
+  RObject Expr = pl2r(A1, names, vars) ;
+  RObject Res = Expr ;
+  try
+  {
+    Language id("identity") ;
+    id.push_back(Expr) ;
+    Res = id.eval() ;
+  }
+  
+  catch(std::exception& ex)
+  {
+    throw PlException(PlCompound("r_eval", PlTermv(A1, PlTerm_atom(ex.what())))) ;
+  }
+
+  PlTerm_var pl ;
+  try
+  {
+    PlCheckFail(pl.unify_term(r2pl(Res, names, vars))) ;
+  }
+  
+  catch(std::exception& ex)
+  {
+    throw PlException(PlCompound("r_eval", PlTermv(A1, PlTerm_atom(ex.what())))) ;
+  }
+
+  return A2.unify_term(pl) ;
+}
+
 // The SWI system should not be initialized twice; therefore, we keep track of
 // its status.
 bool pl_initialized = false ;
@@ -1020,7 +1219,7 @@ LogicalVector init_(String argv0)
   argv[0] = argv0.get_cstring() ;
   argv[1] = "-q" ;
   if(!PL_initialise(argc, (char**) argv))
-    stop("rswipl: initialization failed.") ;
+    stop("rswipl_init: initialization failed.") ;
 
   pl_initialized = true ;  
   return true ;
@@ -1031,12 +1230,14 @@ LogicalVector done_()
 {
   if(!pl_initialized)
   {
-    warning("rswipl: swipl has not been initialized") ;
+    warning("rswipl_done: swipl has not been initialized") ;
     return true ;
   }
 
+  // Just in case there are open queries
+  clear_() ;
+
   PL_cleanup(0) ;
   pl_initialized = false ;
   return true ;
 }
-
