@@ -431,7 +431,7 @@ PLX_WRAP(int                     , release_stream_noerror          , (IOSTREAM *
 PLX_WRAP(IOSTREAM *              , open_resource                   , (module_t m, const char *name, const char *rc_class, const char *mode), (m, name, rc_class, mode))
 
 // (skipped):: IOSTREAM **_PL_streams(void);	/* base of streams */
-PLX_EXCE(int                     , write_term                      , (IOSTREAM *s, term_t term, int precedence, int flags), (s, term, precedence, flags))
+PLX_ASIS(int                     , write_term                      , (IOSTREAM *s, term_t term, int precedence, int flags), (s, term, precedence, flags))
 PLX_ASIS(bool                    , ttymode                         , (IOSTREAM *s), (s))
 
 // TODO: PL_put_term_from_chars depends on CVT_EXCEPTION - ? make version that checks this and throws an exception?
@@ -512,11 +512,13 @@ PLX_EXCE(int                     , thread_destroy_engine           , (), ())
 PLX_ASIS(int                     , thread_at_exit                  , (void (*function)(void *), void *closure, int global), (function, closure, global))
 PLX_ASIS(int                     , thread_raise                    , (int tid, int sig), (tid, sig))
 
-#if defined(_WINDOWS_) || defined(_WINDOWS_H) /* <windows.h> is included */
+// JW: disabled.  Claims these functions are not present in Windows, blocking the build.
+#if 0 && defined(_WINDOWS_) || defined(_WINDOWS_H) /* <windows.h> is included */
 PLX_ASIS(int                     , w32thread_raise                 , (DWORD dwTid, int sig), (dwTid, sig))
 PLX_ASIS(int                     , wait_for_console_input          , (void *handle), (handle))
 PLX_ASIS(int                     , w32_wrap_ansi_console           , (), ())
 PLX_ASIS(const char*             , w32_running_under_wine          , (), ())
+PLX_EXCE(LRESULT                 , win_message_proc                , (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam), (hwnd, message, wParam, lParam))
 #endif
 
 PLX_ASIS(PL_engine_t             , create_engine                   , (PL_thread_attr_t *attributes), (attributes))
@@ -531,10 +533,6 @@ PLX_VOID(void                    , prof_exit                       , (void *node
 PLX_ASIS(int                     , prolog_debug                    , (const char *topic), (topic))
 PLX_ASIS(int                     , prolog_nodebug                  , (const char *topic), (topic))
 
-#if defined(_WINDOWS_) || defined(_WINDOWS_H) /* <windows.h> is included */
-PLX_EXCE(LRESULT                 , win_message_proc                , (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam), (hwnd, message, wParam, lParam))
-#endif
-
 // (skipped):: int _PL_get_xpce_reference(term_t t, xpceref_t *ref);
 // (skipped):: int _PL_unify_xpce_reference(term_t t, xpceref_t *ref);
 // (skipped):: int _PL_put_xpce_reference_i(term_t t, uintptr_t r);
