@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2020, University of Amsterdam
+    Copyright (c)  1997-2023, University of Amsterdam
 			      VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -111,6 +112,7 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
   int do_throw = FALSE;
   fid_t fid;
   int rc;
+  int msg_rep = REP_UTF8;
 
   if ( exception_term )			/* do not overrule older exception */
     return FALSE;
@@ -128,6 +130,7 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
   { if ( errno == EPLEXCEPTION )
       return FALSE;
     msg = OsError();
+    msg_rep = REP_MB;
   }
 
   LD->exception.processing = TRUE;	/* allow using spare stack */
@@ -776,7 +779,7 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
     }
 
     if ( rc && msg )
-    { rc = PL_put_atom_chars(msgterm, msg);
+    { rc = PL_put_chars(msgterm, PL_ATOM|msg_rep, (size_t)-1, msg);
     }
 
     if ( rc )
