@@ -145,7 +145,7 @@ DbgPrintInstruction(LocalFrame FR, Code PC)
   { GET_LD
 
     if ( ofr != FR )
-    { Sfprintf(Serror, "#%zd at [%ld] predicate %s\n",
+    { Sfprintf(Serror, "#%zd at [%ud] predicate %s\n",
 	       loffset(FR),
 	       levelFrame(FR),
 	       predicateName(FR->predicate));
@@ -213,8 +213,8 @@ updateAlerted(PL_local_data_t *ld)
 #ifdef O_PROFILE
   if ( ld->profile.active )			mask |= ALERT_PROFILE;
 #endif
-#ifdef O_PLMT
-  if ( ld->exit_requested )			mask |= ALERT_EXITREQ;
+#ifdef O_ENGINES
+  if ( ld->thread.exit_requested )		mask |= ALERT_EXITREQ;
 #endif
 #ifdef O_LIMIT_DEPTH
   if ( ld->depth_info.limit != DEPTH_NO_LIMIT ) mask |= ALERT_DEPTHLIMIT;
@@ -344,6 +344,8 @@ fid_t
 PL_open_foreign_frame(DECL_LD)
 { size_t lneeded = sizeof(struct fliFrame) + MINFOREIGNSIZE*sizeof(word);
 
+  if ( LD->atoms.gc_active )
+    return 0;
   if ( !ensureLocalSpace(lneeded) )
     return 0;
 
@@ -1488,7 +1490,7 @@ do_undo(mark *m)
    our block is not the real memory pointer.
 */
 
-#ifdef O_PLMT
+#ifdef O_ENGINES
 #define localDefinition(def) LDFUNC(localDefinition, def)
 static Definition
 localDefinition(DECL_LD Definition def)
@@ -1533,7 +1535,7 @@ destroyLocalDefinition(Definition def, unsigned int tid)
 Definition
 getLocalProcDefinition(DECL_LD Definition def)
 {
-#ifdef O_PLMT
+#ifdef O_ENGINES
   if ( true(def, P_THREAD_LOCAL) )
   { MEMORY_ACQUIRE();
     return localDefinition(def);
@@ -2767,6 +2769,8 @@ PL_cut_query(qid_t qid)
   { WITH_LD(qid->engine)
     { QueryFrame qf = QueryFromQid(qid);
 
+      if ( LD->query != qf )
+	return PL_S_NOT_INNER;
       DEBUG(0, assert(qf->magic == QID_MAGIC));
       if ( qf->foreign_frame )
 	PL_close_foreign_frame(qf->foreign_frame);
@@ -2799,6 +2803,8 @@ PL_close_query(qid_t qid)
   { WITH_LD(qid->engine)
     { QueryFrame qf = QueryFromQid(qid);
 
+      if ( LD->query != qf )
+	return PL_S_NOT_INNER;
       DEBUG(0, assert(qf->magic == QID_MAGIC));
       if ( qf->foreign_frame )
 	PL_close_foreign_frame(qf->foreign_frame);
@@ -2930,7 +2936,7 @@ struct register_file;
  * macros (and from functions with local __PL_ld defined), so replicating
  * the pl-builtin.h logic here isn't such a hazard.
  */
-#  if (defined(O_PLMT) || defined(O_MULTIPLE_ENGINES)) && USE_LD_MACROS
+#  if (defined(O_PLMT) || defined(O_ENGINES)) && USE_LD_MACROS
 #   define VMI_ARG_DECL PL_local_data_t *__PL_ld, Code PC, struct register_file *registers
 #   define VMI_ARG_PASS __PL_ld, PC, registers
 #  else
@@ -3239,6 +3245,8 @@ depart_continue() to do the normal thing or to the backtrack point.
   QF  = QueryFromQid(qid);
   if ( QF->magic == QID_CMAGIC )
     return FALSE;
+  if ( LD->query != QF /*|| (void*)fli_context > (void*)QF*/ )
+    return PL_S_NOT_INNER;
   DEBUG(CHK_SECURE, assert(QF->magic == QID_MAGIC));
   if ( true(QF, PL_Q_DETERMINISTIC) )	/* last one succeeded */
   { fid_t fid = QF->foreign_frame;
